@@ -1,12 +1,12 @@
 /*-------------------------------------------------------------------------
  *
- *	  ISO8859_1 <--> UTF-8
+ *	  ISO8859_1 <--> UTF8
  *
  * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/utils/mb/conversion_procs/utf8_and_iso8859_1/utf8_and_iso8859_1.c,v 1.10.4.1 2006/05/21 20:06:17 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/mb/conversion_procs/utf8_and_iso8859_1/utf8_and_iso8859_1.c,v 1.12.2.2 2009/01/29 19:24:37 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -35,14 +35,12 @@ extern Datum utf8_to_iso8859_1(PG_FUNCTION_ARGS);
 Datum
 iso8859_1_to_utf8(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned short c;
 
-	Assert(PG_GETARG_INT32(0) == PG_LATIN1);
-	Assert(PG_GETARG_INT32(1) == PG_UTF8);
-	Assert(len >= 0);
+	CHECK_ENCODING_CONVERSION_ARGS(PG_LATIN1, PG_UTF8);
 
 	while (len > 0)
 	{
@@ -67,15 +65,13 @@ iso8859_1_to_utf8(PG_FUNCTION_ARGS)
 Datum
 utf8_to_iso8859_1(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned short c,
 				c1;
 
-	Assert(PG_GETARG_INT32(0) == PG_UTF8);
-	Assert(PG_GETARG_INT32(1) == PG_LATIN1);
-	Assert(len >= 0);
+	CHECK_ENCODING_CONVERSION_ARGS(PG_UTF8, PG_LATIN1);
 
 	while (len > 0)
 	{
